"""Add Comment.links

Revision ID: 5b980f0dc365
Revises: 1fab8c420678
Create Date: 2024-05-06 16:42:35.512933

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "5b980f0dc365"
down_revision = "1fab8c420678"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("comment", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("links", sa.ARRAY(sa.String()), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("comment", schema=None) as batch_op:
        batch_op.drop_column("links")

    # ### end Alembic commands ###
