"""add mimetype column to attachment

Revision ID: c68c2a62cfac
Revises: 0ec3762a745d
Create Date: 2020-04-12 22:34:50.597925

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c68c2a62cfac"
down_revision = "0ec3762a745d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "attachment_file",
        sa.Column("mimetype", sa.String(length=255), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("attachment_file", "mimetype")
    # ### end Alembic commands ###
