"""empty message

Revision ID: df1834485f57
Revises: 9a09467f9b2c
Create Date: 2019-04-12 15:54:48.151083

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "df1834485f57"
down_revision = "9a09467f9b2c"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "custom_action", sa.Column("is_ajax", sa.Boolean(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("custom_action", "is_ajax")
    # ### end Alembic commands ###
