"""Add casting label

Revision ID: d8dcd5196d57
Revises: 7bc746997e8d
Create Date: 2019-08-07 17:14:22.968405

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "d8dcd5196d57"
down_revision = "7bc746997e8d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "entity_link", sa.Column("label", sa.String(length=80), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("entity_link", "label")
    # ### end Alembic commands ###
