"""empty message

Revision ID: 43d0cf0ed5e7
Revises: 40dea9555940
Create Date: 2019-05-15 22:36:39.136039

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "43d0cf0ed5e7"
down_revision = "40dea9555940"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "preview_file",
        sa.Column("original_name", sa.String(length=250), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("preview_file", "original_name")
    # ### end Alembic commands ###
